@@ -9,7 +9,7 @@ struct PatchDiffer {
 impl PatchDiffer {
     fn new() -> Self {
         Self {
-            path: "/".to_string(),
+            path: "".to_string(),
             patch: super::Patch(Vec::new()),
             shift: 0,
         }
@@ -19,9 +19,7 @@ impl PatchDiffer {
 impl<'a> treediff::Delegate<'a, treediff::value::Key, Value> for PatchDiffer {
     fn push(&mut self, key: &treediff::value::Key) {
         use std::fmt::Write;
-        if self.path.len() != 1 {
-            self.path.push('/');
-        }
+        self.path.push('/');
         match *key {
             treediff::value::Key::Index(idx) => write!(self.path, "{}", idx - self.shift).unwrap(),
             treediff::value::Key::String(ref key) => append_path(&mut self.path, key),
@@ -29,10 +27,7 @@ impl<'a> treediff::Delegate<'a, treediff::value::Key, Value> for PatchDiffer {
     }
 
     fn pop(&mut self) {
-        let mut pos = self.path.rfind('/').unwrap_or(0);
-        if pos == 0 {
-            pos = 1;
-        }
+        let pos = self.path.rfind('/').unwrap_or(0);
         self.path.truncate(pos);
         self.shift = 0;
     }
@@ -94,10 +89,8 @@ fn append_path(path: &mut String, key: &str) {
 ///
 /// ```rust
 /// #[macro_use]
-/// extern crate serde_json;
-/// extern crate json_patch;
-///
-/// use json_patch::{patch, diff, from_value};
+/// use json_patch::{Patch, patch, diff};
+/// use serde_json::{json, from_value};
 ///
 /// # pub fn main() {
 /// let left = json!({
@@ -121,7 +114,7 @@ fn append_path(path: &mut String, key: &str) {
 /// });
 ///
 /// let p = diff(&left, &right);
-/// assert_eq!(p, from_value(json!([
+/// assert_eq!(p, from_value::<Patch>(json!([
 ///   { "op": "remove", "path": "/author/familyName" },
 ///   { "op": "remove", "path": "/tags/1" },
 ///   { "op": "replace", "path": "/title", "value": "Hello!" },
@@ -142,7 +135,7 @@ pub fn diff(left: &Value, right: &Value) -> super::Patch {
 
 #[cfg(test)]
 mod tests {
-    use serde_json::Value;
+    use serde_json::{json, Value};
 
     #[test]
     pub fn replace_all() {
@@ -151,10 +144,29 @@ mod tests {
         assert_eq!(
             p,
             serde_json::from_value(json!([
-                { "op": "replace", "path": "/", "value": null },
+                { "op": "replace", "path": "", "value": null },
             ]))
             .unwrap()
         );
+        let mut left = json!({"title": "Hello!"});
+        crate::patch(&mut left, &p).unwrap();
+    }
+
+    #[test]
+    pub fn diff_empty_key() {
+        let left = json!({"title": "Something", "": "Hello!"});
+        let right = json!({"title": "Something", "": "Bye!"});
+        let p = super::diff(&left, &right);
+        assert_eq!(
+            p,
+            serde_json::from_value(json!([
+                { "op": "replace", "path": "/", "value": "Bye!" },
+            ]))
+            .unwrap()
+        );
+        let mut left_patched = json!({"title": "Something", "": "Hello!"});
+        crate::patch(&mut left_patched, &p).unwrap();
+        assert_eq!(left_patched, right);
     }
 
     #[test]
@@ -164,7 +176,7 @@ mod tests {
         assert_eq!(
             p,
             serde_json::from_value(json!([
-                { "op": "replace", "path": "/", "value": { "title": "Hello!" } },
+                { "op": "replace", "path": "", "value": { "title": "Hello!" } },
             ]))
             .unwrap()
         );
@@ -218,15 +230,13 @@ mod tests {
     #[test]
     fn escape_json_keys() {
         let mut left = json!({
-            "/slashed/path": 1
+            "/slashed/path/with/~": 1
         });
         let right = json!({
-            "/slashed/path": 2,
+            "/slashed/path/with/~": 2,
         });
         let patch = super::diff(&left, &right);
 
-        eprintln!("{:?}", patch);
-
         crate::patch(&mut left, &patch).unwrap();
         assert_eq!(left, right);
     }
